#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "cpuboot.h"

#define BULK_TIMEOUT (10000)

uint8_t perf_boot_msg[4] = {0x02, 0x00, 0x03, 0xF0};
uint8_t perf_boot_size[4]; // patched by load_image_file

static void
dump_omap_asic_id(omap_asic_id *id)
{
	fprintf(stderr, "OMAP ASIC ID\n");
	fprintf(stderr, "  Items:               %d\n", id->items);
	fprintf(stderr, " --\n");
	fprintf(stderr, "  ID.header:           0x%02x, 0x%02x, 0x%02x\n",
			id->id.header.id, id->id.header.length, id->id.header.fixed);
	fprintf(stderr, "  ID.device:           0x%02x%02x%02x\n",
			id->id.device[0], id->id.device[1], id->id.device[2]);
	fprintf(stderr, "  ID.version:          %d\n", id->id.version);	
	fprintf(stderr, " --\n");
	fprintf(stderr, "  SecureMode.header:   0x%02x, 0x%02x, 0x%02x\n",
			id->secure_mode.header.id, id->secure_mode.header.length, id->secure_mode.header.fixed);
	fprintf(stderr, "  SecureMode.data:     %d\n", id->secure_mode.data);
	fprintf(stderr, " --\n");
}

static void
patch_omap_message(void)
{
	/* patch the omap boot size message */
	perf_boot_size[0] = ImageSize & 0xFF;
	perf_boot_size[1] = (ImageSize >> 8) & 0xFF;
	perf_boot_size[2] = (ImageSize >> 16) & 0xFF;
	perf_boot_size[3] = (ImageSize >> 24) & 0xFF;
}

int omap_load_firmware_usb(usb_dev_handle *hdl)
{
	int rc;

	if (usb_set_configuration(hdl, 1)) {
		fprintf(stderr, "Unable to set device configuration.\n");
		return -1;
	}
	if (usb_claim_interface(hdl, 0)) {
		fprintf(stderr, "Unable to claim interface for the device.\n");
		return -1;
	}

	omap_asic_id asic_id;
	memset(&asic_id, 0, sizeof(asic_id));

	rc = usb_bulk_read(hdl, 0x1, (char *)&asic_id, sizeof(asic_id), BULK_TIMEOUT);
	if (rc < 0) {
		fprintf(stderr, "Unable to build read the asic_id.\n");
		return rc;
	}

	if (Verbose) {
		dump_omap_asic_id(&asic_id);
	}

	patch_omap_message();

	rc = usb_bulk_write(hdl, 0x1, (char *)perf_boot_msg, sizeof(perf_boot_msg), BULK_TIMEOUT);
	if (rc < 0) {
		fprintf(stderr, "Unable to bulk write the boot message: %d\n", errno);
		return rc;
	}

	rc = usb_bulk_write(hdl, 0x1, (char *)perf_boot_size, sizeof(perf_boot_size), BULK_TIMEOUT);
	if (rc < 0) {
		fprintf(stderr, "Unable to bulk write te image size: %d\n", errno);
		return rc;
	}

	int count = 0;
	while (count < ImageSize) {
		int amount = (count + 1024 < ImageSize) ? 1024 : (ImageSize - count);
		rc = usb_bulk_write(hdl, 0x1, (char *)ImageData + count, amount, BULK_TIMEOUT);
		if (rc < 0) {
			fprintf(stderr, "Unable to bulk write the payload. (amount %d, count %d, size %d)\n",
					amount, count, ImageSize);
			return -1;
		}
		count += amount;
	}

	return count;
}

int omap_load_firmware_serial(int fd)
{
	/* Load the firmware over the UART */
	unsigned char b = 0xFF;
	int count;
	int spinner;
	int rc;

	fprintf(stderr, "Waiting for device on UART...\n");			

	omap_asic_id asic_id;
	memset(&asic_id, 0, sizeof(asic_id));

	/* Wait for a matching first byte over the UART */
	do {
		rc = read(fd, &b, 1);
		if (rc < 0) {
			fprintf(stderr, "Unable to read ID byte from UART.\n");
			return rc;
		}
	} while (b != 0x4);

	fprintf(stderr, "Found an OMAP3430 in UART Boot Mode!\n");
	asic_id.items = b;

	/* UART boot mode doesn't sent a checksum block (Block Count of 4) */
	read_bytes(fd, (uint8_t *)&asic_id.id, sizeof(asic_id) - 1 - sizeof(asic_id.checksum));

	patch_omap_message();

	/* Send the command to perform UART boot and the size of the image following */
	write(fd, perf_boot_msg, sizeof(perf_boot_msg));
	write(fd, perf_boot_size, sizeof(perf_boot_size));

	if (Verbose) {
		/* Safe to print this out now that the boot will be waiting for data */
		dump_omap_asic_id(&asic_id);
	}

	count = 0;
	spinner = 0;
	while (count < ImageSize) {
		int amount = (count + 4 < ImageSize) ? 4 : (ImageSize - count);
		if (write(fd, ImageData + count, amount) != amount) {
			fprintf(stderr, "Failed to write to device: %s\n", strerror(errno));
			return -1;
		}
		count += amount;
		spinner ++;
		if (spinner > 16) {
			fprintf(stderr, "%d of %d bytes sent\r", count, ImageSize);
			spinner = 0;
		}
		usleep(1000);
	}

	return count;
}

